@@ -2,7 +2,7 @@ import os
 
 
 def get_config_dir() -> str:
-    root = os.environ.get('XDG_CONFIG_HOME') or os.path.join(
+    root = os.environ.get('GITA_PROJECT_HOME') or os.environ.get('XDG_CONFIG_HOME') or os.path.join(
         os.path.expanduser('~'), '.config')
     return os.path.join(root, "gita")
 
